@@ -7,7 +7,7 @@ Panda::XS - useful features and typemaps for XS modules.
 
 =cut
 
-our $VERSION = '1.2.0';
+our $VERSION = '1.3.0';
 require Panda::XSLoader;
 Panda::XSLoader::load();
 
@@ -1358,37 +1358,39 @@ API is the same as for std::shared_ptr.
 
 All functions and types are in 'xs' namespace so that you will actually need C++ to use them.
 
+Functions marked with [pTHX] must receive aTHX as a first arg.
+
 =head4 panda::dynamic_pointer_cast/const_pointer_cast/static_pointer_cast
 
 The same as std::*_pointer_cast for panda::shared_ptr and std::shared_ptr.
 
 =head2 PAYLOAD FUNCTIONS
 
-=head4 void sv_payload_attach (SV* sv, void* ptr, const payload_marker_t* marker = NULL)
+=head4 void sv_payload_attach (SV* sv, void* ptr, const payload_marker_t* marker = NULL) [pTHX]
 
 Attach payload ptr to sv. Markers allow you to store multiple payloads in a single SV.
 Marker doesn't need to be initialized somehow, its just a valid memory pointer. Usually its a static global variable.
 If no marker provided, default one is used (global to the whole program).
 
-=head4 bool sv_payload_exists (const SV* sv, const payload_marker_t* marker = NULL)
+=head4 bool sv_payload_exists (const SV* sv, const payload_marker_t* marker = NULL) [pTHX]
 
 Returns true if sv has any payload for marker.
 
-=head4 void* sv_payload (const SV* sv, const payload_marker_t* marker = NULL)
+=head4 void* sv_payload (const SV* sv, const payload_marker_t* marker = NULL) [pTHX]
 
 Returns payload attached to sv.
 
-=head4 int sv_payload_detach (SV* sv, const payload_marker_t* marker = NULL)
+=head4 int sv_payload_detach (SV* sv, const payload_marker_t* marker = NULL) [pTHX]
 
 Removes payload from sv and returns number of payloads removed.
 
-=head4 void rv_payload_attach (const SV* rv, void* ptr, const payload_marker_t* marker = NULL)
+=head4 void rv_payload_attach (const SV* rv, void* ptr, const payload_marker_t* marker = NULL) [pTHX]
 
-=head4 bool rv_payload_exists (const SV* rv, const payload_marker_t* marker = NULL)
+=head4 bool rv_payload_exists (const SV* rv, const payload_marker_t* marker = NULL) [pTHX]
 
-=head4 void* rv_payload (const SV* rv, const payload_marker_t* marker = NULL)
+=head4 void* rv_payload (const SV* rv, const payload_marker_t* marker = NULL) [pTHX]
 
-=head4 int rv_payload_detach (const SV* rv, const payload_marker_t* marker = NULL)
+=head4 int rv_payload_detach (const SV* rv, const payload_marker_t* marker = NULL) [pTHX]
 
 Same as sv_* but operates on sv referenced by a given rv. Doesn't check if a given RV is valid.
 
@@ -1417,7 +1419,7 @@ threaded perls.
 
 =head2 SUPER/NEXT METHOD FUNCTIONS
 
-=head4 SV* call_super (CV* cv, SV** args, I32 items, I32 flags = 0)
+=head4 SV* call_super (CV* cv, SV** args, I32 items, I32 flags = 0) [pTHX]
 
 Calls super method from inside XS method. 'cv' is a pointer to currently running XS function (every XS function has this variable).
 'args' and 'items' describes perl variables to pass to super method. If you want to passthrough all the arguments your XS function
@@ -1425,15 +1427,15 @@ received, pass '&ST(0)' and 'items'. 'flags' is passed to Perl's API call_method
 context. Returned SV* has its refcounter increased. If you don't need it, decrement it. If you don't need return value at all,
 pass G_DISCARD in flags.
 
-=head4 SV* call_next_method (CV* cv, SV** args, I32 items, I32 flags = 0)
+=head4 SV* call_next_method (CV* cv, SV** args, I32 items, I32 flags = 0) [pTHX]
 
 Calls next method (C3 MRO). See "call_super" for argument details.
 
-=head4 SV* call_next_maybe (CV* cv, SV** args, I32 items, I32 flags = 0)
+=head4 SV* call_next_maybe (CV* cv, SV** args, I32 items, I32 flags = 0) [pTHX]
 
 Calls next method (C3 MRO). Return undef if no next method. See "call_super" for argument details.
 
-=head4 SV* call_next (CV* cv, SV** args, I32 items, next_t type, I32 flags = 0)
+=head4 SV* call_next (CV* cv, SV** args, I32 items, next_t type, I32 flags = 0) [pTHX]
 
 Generic form of call_super/call_next_method/call_next_maybe. 'type' is either xs::NEXT_SUPER or xs::NEXT_METHOD or xs::NEXT_MAYBE.
 
@@ -1531,6 +1533,8 @@ otherwise from $target itself.
 
 =head1 THREADED PERLS SUPPORT
 
+=head2 HOOKING THREAD CREATION FOR XS OBJECTS
+
 If you don't do anything special, on threaded perls after thread creation you will most likely core dump.
 The reason is that when perl creates a thread it duplicates all perl variables available in current interpreter.
 So that you will likely get 2 perl variables holding a pointer to the case C/C++ object (refcnt won't help - nobody calls C<retain()>).
@@ -1573,7 +1577,7 @@ For example:
 To do this, don't define C<CLONE_SKIP> method, instead set C<on_svdup> parameter of T_OEXT class to the name of your callback which
 will handle thread creation for objects of your class. Callback must have the following signature:
 
-    <basetype> <callbackname> (<basetype> arg);
+    <basetype> <callbackname> (pTHX_ <basetype> arg);
     
 Callback will be passed current C object. Callback shall return a C object to be set to the duplicated perl variable
 (it may be a new one or the same object). Keep in mind that if your module shares API for other modules, this callback MUST be
@@ -1582,10 +1586,10 @@ your callback name into those XS functions. Don't forget to specify it with full
 
     # in .h
     namespace my {
-        MyClass* myclass_onsvdup (MyClass* old);
+        MyClass* myclass_onsvdup (pTHX_ MyClass* old);
     }
     # in .cc
-    MyClass* my::myclass_onsvdup (MyClass* old) {
+    MyClass* my::myclass_onsvdup (pTHX_ MyClass* old) {
         return new MyClass(old->data);
     }
     
@@ -1594,6 +1598,99 @@ your callback name into those XS functions. Don't forget to specify it with full
     
 =over
 
+=head2 aTHX/pTHX politics and helpers
+
+Perls has 2 options for XS when working on threaded perls:
+
+1) Not defining PERL_NO_GET_CONTEXT. Advantage is that you don't have to put pTHX to every function's signature and don't have to
+pass it to every function which works with perl variables. Disadvatage is - decreased performance on threaded perls.
+
+2) Defining PERL_NO_GET_CONTEXT.
+
+Unfortunatelly, as Panda::Install/Panda::XS concept is all about sharing C code, modules that use Panda::XS cannot have different
+PERL_NO_GET_CONTEXT state. Because in that case, binary incompability between function signatures would accure.
+
+Panda::XS strictly sets PERL_NO_GET_CONTEXT options, so that you have generally to accept pTHX and pass aTHX to all functions that
+work with perl variables. In this case you'll achieve maximum performance on threaded perls.
+
+However if you don't want to write pTHX/aTHX every time, you can write
+
+    dTHX;
+    
+at the top of those functions and actually saying revert to behaviour without PERL_NO_GET_CONTEXT (but you still need to pass aTHX
+to functions which are defined with pTHX, unless called via macros - like perl's API).
+
+For example, you can't receive pTHX in C++ objects destructors, so the options for you are either write C<dTHX;> on the top of
+destructor or save perl interpreter object in constructor of your object to property named 'my_perl', or use C<using xs::my_perl>
+global interpreter.
+
+One more problem is static initialization, for example:
+
+    # my.h
+    extern SV* myname;
+    # my.cc
+    #include <my.h>
+    SV* myname = newSVpv("john", 0);
+    
+This won't work on threaded perls with PERL_NO_GET_CONTEXT because no one defined C<my_perl>. You shouldn't say C<dTHX;> in the
+global-scoped code. This problem can be solved using C<using xs::my_perl>.
+
+=head3 HELPERS
+
+=head4 xs::my_perl
+
+This variable holds correct perl interpreter for each thread. Just say C<using xs::my_perl> and you'll get my_perl interpreter
+for the rest of your code's scope.
+
+Keep in mind that this C<my_perl> variable is get from thread-local storage, like C<dTHX>, so this is actually a convenince tool,
+reverting to performance as PERL_NO_GET_CONTEXT has not been defined. If you want maximum performance on threaded perls, use
+pTHX/aTHX everywhere. You could however use mixed style, saying C<using xs::my_perl> and passing pTHX only for performance-critical
+functions.
+
+This variable should not be used on non-threaded perls (this is not a problem as aTHX is empty in that case). However it still
+exists on non-threaded perls, so that you can say C<using xs::my_perl> without wrapping it into C<#ifdef PERL_IMPLICIT_CONTEXT>.
+
+=head4 mTHX
+
+Another way to achieve maximum perfomance on threaded-perls for C++ objects is to hold my_perl as class variable. In this case
+you don't have to receive pTHX in every method of your object and don't even have to say C<dTHX> in desctructor.
+
+This helper defines this class member (it defines nothing on non-threaded perls). Example
+
+    class MyXSClass {
+        int val;
+        mTHX;
+        MyXSClass (int val) : val(val) {
+            aTHXa(PERL_GET_THX);
+        }
+        // pTHX is not needed anymore in method's definitions.
+    }
+
+=head4 mTHXa(interpreter)
+
+Sometimes you have to pass aTHX to constructors of your class members and therefore setting C<aTHXa> in constructor's code is too
+late.
+
+C<mTHXa(interpreter)> initializes class member C<my_perl> with value C<interpreter> in initializer list.
+On threaded perls it is C<my_perl(interpreter),>, otherwise empty. Example:
+
+    class MyXSClass {
+        int val;
+        MySVContainer container;
+        mTHX;
+        MyXSClass (int val) : val(val), mTHXa(PERL_GET_THX) container(aTHX) {
+            ...
+        }
+    }
+
+Note that you should not write C<,> after mTHXa. Therefore it cannot be the last on the list.
+
+=head3 TIP
+
+If you don't care about performance on threaded perls then the most convenient solution is C<using xs::my_perl> on the top of the
+file. In this case you don't have to put pTHX in every function's definition. However you still have to pass aTHX to functions which
+defined with pTHX.
+
 =head1 CAVEATS
 
 If module A binary depends on module B and module B updates and becomes binary incompatible, undefined behaviour may happen.
