@@ -63,12 +63,10 @@ typedef IO OIO;
 
 #ifdef aTHX // define class member helpers for storing perl interpreter
 #  define mTHX      pTHX;
-#  define mTHXa(a)  aTHX(a)
-#  define mTHXa_(a) aTHX(a),
+#  define mTHXa(a)  aTHX(a),
 #else
 #  define mTHX
 #  define mTHXa(a)
-#  define mTHXa_(a)
 #endif
 
 namespace xs {
@@ -79,7 +77,17 @@ enum next_t {
     NEXT_MAYBE  = 2
 };
 
-//THREAD_LOCAL PerlInterpreter* my_perl;
+struct my_perl_auto_t { // per-thread interpreter to help dealing with pTHX/aTHX, especially for static initialization
+    operator PerlInterpreter*() const {
+#ifdef PERL_IMPLICIT_CONTEXT
+        static THREAD_LOCAL PerlInterpreter* _i = PERL_GET_THX;
+        return _i;
+#else
+        return NULL;
+#endif
+    }
+};
+extern my_perl_auto_t my_perl;
 
 typedef int (*on_svdup_t) (pTHX_ MAGIC* mg, CLONE_PARAMS* param);
 
